/*******************************************************************************
 * Module      : $RCSfile: callbacks.c,v $
 * Description : Callback functions of Gnome-ToDo
 *             : 
 * Date        : $Date: 2001/05/23 13:58:38 $
 * Version     : $Revision: 1.5 $
 * Author      : Chris Hall
 * Identifier  : $Id: callbacks.c,v 1.5 2001/05/23 13:58:38 chall Exp $
 ********************************************************************************
 * Change History
 *
 * $Log: callbacks.c,v $
 * Revision 1.5  2001/05/23 13:58:38  chall
 * Added GPL license text and installation instructions
 *
 * Revision 1.4  2001/05/23 12:53:16  chall
 * Fixed bug, can now find help pages
 *
 * Revision 1.3  2001/05/23 12:30:00  chall
 * Release 0.1
 *
 ********************************************************************************
 *
 * Gnome ToDo - Task reminder program for Gnome
 * Copyright (C) 2001 Chris Hall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Chris Hall Chris@clapham.org
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "jobnode.h"
#include <time.h>
#include <stdio.h>
#include <pwd.h>
#include <sys/stat.h>
#include <sys/types.h>

/* #define CFGFILE   "/home/chall/Projects/GnomeTD/src/config" */

#define CFGFILE   "./config"

#define DEFAULT_FONT "-misc-fixed-medium-r-*-*-13-*-*-*-*-*-*-*"

static GtkWidget *window1        = (GtkWidget *) NULL;
static GtkWidget *dialog1        = (GtkWidget *) NULL;
static GtkWidget *about2         = (GtkWidget *) NULL;
static GtkWidget *fileselection1 = (GtkWidget *) NULL;
extern GtkWidget *app1;
GtkWidget *propertybox1 = (GtkWidget *) NULL;

static JobTree *T;



/* Papercut - Linkin Park */

void LoadMainWindow(void);
gint UpdateRemaining(gpointer *);
gint UpdateDaily(gpointer *);
gint RebuildCList(void);
void RunJob(JobNode *);
void app_destroy(void);
int  readtree(char *);
int  writetree(char *);
void fill_date_example(void);
void cfgfile_save(void);
void update_statusbar(void);



static gint alarmtag = 0;
gint   JobAction(gpointer *);
void   SetAlarm(void);
static gint rowsel = -1;
static JobNode *jn = (JobNode *) NULL;



typedef struct __settings {
    char     fname[1024];
    char     dtfmt[128];
    GdkFont  *fnt;
    gchar    fntc[128];
    GdkColor warned;
    GdkColor expired;
    GdkColor normal;
    GdkColor background;
    gint     colwid[7];

} settings;

settings Set;

void
on_new_file1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    on_newfile_clicked ((GtkButton *) NULL, (gpointer ) NULL);
}


void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    on_file_clicked ((GtkButton *) NULL, (gpointer ) NULL);
}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if ( Set.fname[0] == '\0' )
        on_save_as1_activate((GtkMenuItem *) NULL, (gpointer *) NULL);
    else
        writetree(Set.fname);
}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

    if ( fileselection1 == (GtkWidget *) NULL )
        fileselection1 = create_fileselection1();

    gtk_window_set_title(GTK_WINDOW(fileselection1), "Save As");
    gtk_object_set_user_data(GTK_OBJECT(fileselection1), (gpointer ) "Save As");

    gtk_widget_show(fileselection1);
}


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if ( Set.fname[0] == '\0' )
    {
        if ( fileselection1 == (GtkWidget *) NULL )
            fileselection1 = create_fileselection1();

        gtk_window_set_title(GTK_WINDOW(fileselection1), "Quit - Save As");
        gtk_object_set_user_data(GTK_OBJECT(fileselection1), (gpointer ) "Quit - Save As");

        gtk_widget_show(fileselection1);
    }
    else
    {
        writetree(Set.fname);
        cfgfile_save();
        gtk_main_quit();
    }
}


void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

    propertybox1 = create_propertybox1 ();

    gtk_widget_show(propertybox1);
}


void
on_about3_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if ( about2 == NULL )
        about2 = create_about2 ();

    gtk_widget_show (about2);

}


gint
on_about2_close                        (GnomeDialog     *gnomedialog,
                                        gpointer         user_data)
{
    gtk_widget_hide (about2);
}




void
on_NewTask_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{

    time_t t;
    struct tm *st;

    gchar dstr[64],
          hr[3],
          mi[3];

    GtkEntry *dtentry,
             *hrent,
             *mient;

    GtkWidget *but,
              *f;

    GtkCalendar *cal;

    GtkText *text;

    GtkToggleButton *tb;




    if (window1 == NULL )
        window1 = create_window1 ();

    text = GTK_TEXT ( lookup_widget (window1, "text1") );
    gtk_text_set_point(text, 0);
    gtk_text_forward_delete(text, gtk_text_get_length(text));

    but = lookup_widget(window1, "button22");
    gtk_widget_set_sensitive(but, TRUE); 

    tb = GTK_TOGGLE_BUTTON (lookup_widget(window1, "togglebutton1"));
    f  = lookup_widget(window1, "frame5");
    gtk_toggle_button_set_active(tb, FALSE);
    gtk_widget_set_sensitive(f, FALSE);

    t = time(&t);
    t = (t / 900) * 900 + 900;
    st = localtime(&t);

    cal      = GTK_CALENDAR ( lookup_widget (window1, "calendar2") );
    dtentry  = GTK_ENTRY ( lookup_widget (window1, "entry3") );
    hrent    = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo1"))->entry) );
    mient    = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo3"))->entry) );

    gtk_signal_handler_block_by_func(GTK_OBJECT(cal), GTK_SIGNAL_FUNC(on_calendar2_day_selected), user_data);
    gtk_signal_handler_block_by_func(GTK_OBJECT(hrent), GTK_SIGNAL_FUNC(on_combo_entry1_changed), user_data);
    gtk_signal_handler_block_by_func(GTK_OBJECT(mient), GTK_SIGNAL_FUNC(on_combo_entry3_changed), user_data);

    gtk_calendar_select_month(cal, st->tm_mon, st->tm_year + 1900);
    gtk_calendar_select_day(cal, st->tm_mday);

    strftime((char *) dstr, 64, Set.dtfmt, st);
    gtk_entry_set_text( dtentry, dstr);
    (void ) sprintf((char *) hr, "%02d", st->tm_hour);
    (void ) sprintf((char *) mi, "%02d", st->tm_min);
    gtk_entry_set_text( hrent, hr);
    gtk_entry_set_text( mient, mi);

    gtk_signal_handler_unblock_by_func(GTK_OBJECT(cal), GTK_SIGNAL_FUNC(on_calendar2_day_selected), user_data);
    gtk_signal_handler_unblock_by_func(GTK_OBJECT(hrent), GTK_SIGNAL_FUNC(on_combo_entry1_changed), user_data);
    gtk_signal_handler_unblock_by_func(GTK_OBJECT(mient), GTK_SIGNAL_FUNC(on_combo_entry3_changed), user_data);


    but = GTK_WIDGET( lookup_widget(app1, "button25"));
    gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
    gtk_widget_set_sensitive(but, FALSE);
    gtk_widget_show (window1);

}

void
on_EditTask_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{

    time_t t;
    struct tm *st;

    gchar dstr[64],
          hr[3],
          mi[3],
          fig[10];

    GtkEntry *dtentry,
             *hrent,
             *mient,
             *entry;

    GtkWidget *but;

    GtkToggleButton *tb;

    GtkCList *cl;
    GtkText *text;

    GtkCalendar *cal;


    

    if (rowsel == -1)
        return;

    if (window1 == NULL)
        window1 = create_window1 ();

    cl = GTK_CLIST(lookup_widget (app1, "clist1"));
    text = GTK_TEXT ( lookup_widget( window1, "text1"));
    but = lookup_widget(window1, "button22");

    gtk_widget_set_sensitive( but, FALSE);

    jn = (JobNode *) gtk_clist_get_row_data(cl, rowsel);
    
    t = getjobtime(jn);
    gtk_text_set_point(text, 0);
    gtk_text_forward_delete(text, gtk_text_get_length(text));
    gtk_text_insert(text, (GdkFont *) NULL, (GdkColor *) NULL, (GdkColor *) NULL, getjobdesc(jn), -1);
    st = localtime(&t);

    cal      = GTK_CALENDAR ( lookup_widget (window1, "calendar2") );
    dtentry  = GTK_ENTRY ( lookup_widget (window1, "entry3") );
    hrent    = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo1"))->entry) );
    mient    = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo3"))->entry) );
    tb       = GTK_TOGGLE_BUTTON ( lookup_widget(window1, "togglebutton1"));

    gtk_signal_handler_block_by_func(GTK_OBJECT(cal), GTK_SIGNAL_FUNC(on_calendar2_day_selected), user_data);
    gtk_signal_handler_block_by_func(GTK_OBJECT(hrent), GTK_SIGNAL_FUNC(on_combo_entry1_changed), user_data);
    gtk_signal_handler_block_by_func(GTK_OBJECT(mient), GTK_SIGNAL_FUNC(on_combo_entry3_changed), user_data);

    gtk_calendar_select_month(cal, st->tm_mon, st->tm_year + 1900);
    gtk_calendar_select_day(cal, st->tm_mday);

    strftime((char *) dstr, 64, Set.dtfmt, st);
    gtk_entry_set_text( dtentry, dstr);
    (void ) sprintf((char *) hr, "%02d", st->tm_hour);
    (void ) sprintf((char *) mi, "%02d", st->tm_min);
    gtk_entry_set_text( hrent, hr);
    gtk_entry_set_text( mient, mi);

    gtk_signal_handler_unblock_by_func(GTK_OBJECT(cal), GTK_SIGNAL_FUNC(on_calendar2_day_selected), user_data);
    gtk_signal_handler_unblock_by_func(GTK_OBJECT(hrent), GTK_SIGNAL_FUNC(on_combo_entry1_changed), user_data);
    gtk_signal_handler_unblock_by_func(GTK_OBJECT(mient), GTK_SIGNAL_FUNC(on_combo_entry3_changed), user_data);


    t = t - getwarntime(jn);
    sprintf(fig, "%ld", t / 86400);
    t -= (t / 86400) * 86400;
    hrent = GTK_ENTRY ( lookup_widget (window1, "entry9"));
    gtk_entry_set_text(hrent, fig);

    sprintf(fig, "%ld", t / 3600);
    t -= (t / 3600) * 3600;
    hrent = GTK_ENTRY ( lookup_widget (window1, "entry11"));
    gtk_entry_set_text(hrent, fig);

    sprintf(fig, "%ld", t / 60);
    t -= (t / 60) * 60;
    hrent = GTK_ENTRY ( lookup_widget (window1, "entry10"));
    gtk_entry_set_text(hrent, fig);


    but = GTK_WIDGET( lookup_widget(app1, "NewTask"));
    gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
    gtk_widget_set_sensitive(but, FALSE);

    if ( getrepeat(jn) != off )
    {
        gtk_toggle_button_set_active(tb, TRUE);

        if ( getrepeat(jn) == daily )
        {
            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "radiobutton17"));
            gtk_toggle_button_set_active(tb, TRUE);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton53"));
            if ( getrepeaton(jn, monday) == yes )
                gtk_toggle_button_set_active(tb, TRUE);
            else
                gtk_toggle_button_set_active(tb, FALSE);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton54"));
            if ( getrepeaton(jn, tuesday) == yes )
                gtk_toggle_button_set_active(tb, TRUE);
            else
                gtk_toggle_button_set_active(tb, FALSE);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton55"));
            if ( getrepeaton(jn, wednesday) == yes )
                gtk_toggle_button_set_active(tb, TRUE);
            else
                gtk_toggle_button_set_active(tb, FALSE);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton56"));
            if ( getrepeaton(jn, thursday) == yes )
                gtk_toggle_button_set_active(tb, TRUE);
            else
                gtk_toggle_button_set_active(tb, FALSE);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton57"));
            if ( getrepeaton(jn, friday) == yes )
                gtk_toggle_button_set_active(tb, TRUE);
            else
                gtk_toggle_button_set_active(tb, FALSE);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton58"));
            if ( getrepeaton(jn, saturday) == yes )
                gtk_toggle_button_set_active(tb, TRUE);
            else
                gtk_toggle_button_set_active(tb, FALSE);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton59"));
            if ( getrepeaton(jn, sunday) == yes )
                gtk_toggle_button_set_active(tb, TRUE);
            else
                gtk_toggle_button_set_active(tb, FALSE);


            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton52"));
            if ( getrepeaton(jn, hour) == yes )
            {
                gtk_toggle_button_set_active(tb, TRUE);
                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo8"))->entry) );
                sprintf(hr, "%02d", (int ) getstarton(jn));
                gtk_entry_set_text(entry, hr);

                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo9"))->entry) );
                sprintf(hr, "%02d", (int ) getendon(jn));
                gtk_entry_set_text(entry, hr);
            }
            else
            {
                gtk_toggle_button_set_active(tb, FALSE);
                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo8"))->entry) );
                gtk_entry_set_text(entry, "00");

                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo9"))->entry) );
                gtk_entry_set_text(entry, "23");
            }

        }
        else
        {

            if ( getrepeat(jn) == monthly )
            {
                tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "radiobutton19"));
                gtk_toggle_button_set_active(tb, TRUE);
            }
            else
            {
                tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "radiobutton20"));
                gtk_toggle_button_set_active(tb, TRUE);
            }


            if ( getsameday(jn) == yes )
            {
                tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "radiobutton21"));
                gtk_toggle_button_set_active(tb, TRUE);
            }
            else
            {
                tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "radiobutton22"));
                gtk_toggle_button_set_active(tb, TRUE);

                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo13"))->entry) );
                gtk_entry_set_editable(entry, TRUE);

                switch(getmodtype(jn))
                {
                    case previous:    gtk_entry_set_text(entry, "Previous");
                                      break;

                    case next:        gtk_entry_set_text(entry, "Next");
                                      break;

                    case first:       gtk_entry_set_text(entry, "First");
                                      break;

                    case second:      gtk_entry_set_text(entry, "Second");
                                      break;

                    case third:       gtk_entry_set_text(entry, "Third");
                                      break;

                    case fourth:      gtk_entry_set_text(entry, "Fourth");
                                      break;

                    case last:        gtk_entry_set_text(entry, "Last");
                                      break;

                }

                gtk_entry_set_editable(entry, FALSE);


                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo14"))->entry) );
                gtk_entry_set_editable(entry, TRUE);

                switch(getmodwhen(jn))
                {
                    case weekday:    gtk_entry_set_text(entry, "Weekday");
                                     break;

                    case monday:     gtk_entry_set_text(entry, "Monday");
                                     break;

                    case tuesday:    gtk_entry_set_text(entry, "Tuesday");
                                     break;

                    case wednesday:  gtk_entry_set_text(entry, "Wednesday");
                                     break;

                    case thursday:   gtk_entry_set_text(entry, "Thursday");
                                     break;

                    case friday:     gtk_entry_set_text(entry, "Friday");
                                     break;

                    case saturday:   gtk_entry_set_text(entry, "Saturday");
                                     break;

                    case sunday:     gtk_entry_set_text(entry, "Sunday");
                                     break;

                    default:         break;
                }

                gtk_entry_set_editable(entry, FALSE);


            }

        }

    }
    else
        gtk_toggle_button_set_active(tb, FALSE);

    gtk_widget_show (window1);

    if ( getrepeat(jn) != off )
        gtk_widget_show(window1);
}


void
on_button20_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{

    GtkText *text;
    GtkEntry *entry;
    GtkRadioButton *b;
    GtkToggleButton *tb;

    char *c;
    time_t t = 0;



    if (jn != (JobNode *) NULL)
        DeleteNode(T, jn);

    jn = NewNode(T);

    text = GTK_TEXT ( lookup_widget (window1, "text1") );

    setjobdesc(jn, (char *) gtk_editable_get_chars(GTK_EDITABLE(text), 0, -1));
    gtk_text_set_point(text, 0);
    gtk_text_forward_delete(text, gtk_text_get_length(text));

    entry = GTK_ENTRY ( lookup_widget (window1, "entry9"));
    t += atoi(gtk_entry_get_text(entry)) * 86400;

    entry = GTK_ENTRY ( lookup_widget (window1, "entry11"));
    t += atoi(gtk_entry_get_text(entry)) * 3600;

    entry = GTK_ENTRY ( lookup_widget (window1, "entry10"));
    t += atoi(gtk_entry_get_text(entry)) * 60;

    if ( t < 0 )
        t = 0;

    DateChanged(GTK_CALENDAR ( lookup_widget ( window1, "calendar2")));
    setwarntime(jn, getjobtime(jn) - t);
    setorigdate(jn);

    b = GTK_RADIO_BUTTON( lookup_widget ( window1, "alarmbutton"));
    if (GTK_TOGGLE_BUTTON(b)->active)
        setjobaction(jn, alarmclock);
    else 
    {
        b = GTK_RADIO_BUTTON( lookup_widget ( window1, "timerbutton"));
        if (GTK_TOGGLE_BUTTON(b)->active)
            setjobaction(jn, timer);
        else
            setjobaction(jn, execute);
    }
    
    tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "togglebutton1"));
    if ( tb->active )
    {
        tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "radiobutton17"));
        if ( tb->active == TRUE )
        {
            setrepeat(jn, daily);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton53"));
            if (tb->active == TRUE)
                setrepeaton(jn, monday, yes);
            else
                setrepeaton(jn, monday, no);
            
            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton54"));
            if (tb->active == TRUE)
                setrepeaton(jn, tuesday, yes);
            else
                setrepeaton(jn, tuesday, no);
            
            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton55"));
            if (tb->active == TRUE)
                setrepeaton(jn, wednesday, yes);
            else
                setrepeaton(jn, wednesday, no);
            
            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton56"));
            if (tb->active == TRUE)
                setrepeaton(jn, thursday, yes);
            else
                setrepeaton(jn, thursday, no);
            
            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton57"));
            if (tb->active == TRUE)
                setrepeaton(jn, friday, yes);
            else
                setrepeaton(jn, friday, no);
            
            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton58"));
            if (tb->active == TRUE)
                setrepeaton(jn, saturday, yes);
            else
                setrepeaton(jn, saturday, no);
            
            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton59"));
            if (tb->active == TRUE)
                setrepeaton(jn, sunday, yes);
            else
                setrepeaton(jn, sunday, no);


            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "checkbutton52"));
            if (tb->active == TRUE)
            {
                setrepeaton(jn, hour, yes);
                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo8"))->entry) );
                setstarton(jn, atoi(gtk_entry_get_text(entry)));
                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo9"))->entry) );
                setendon(jn, atoi(gtk_entry_get_text(entry)));
            }
            else
                setrepeaton(jn, hour, no);

        }
        else
        {
            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "radiobutton19"));
            if (tb->active == TRUE)
                setrepeat(jn, monthly);
            else
                setrepeat(jn, yearly);

            tb = GTK_TOGGLE_BUTTON( lookup_widget ( window1, "radiobutton21"));
            if (tb->active == TRUE)
                setsameday(jn, yes);
            else
            {
                setsameday(jn, no);
                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo13"))->entry) );
                c = gtk_entry_get_text(entry);


                if (strncmp("Pr", c, 2) == 0)
                    setmodtype(jn, previous);
                else if (strncmp("Ne", c, 2) == 0)
                    setmodtype(jn, next);
                else if (strncmp("Fi", c, 2) == 0)
                    setmodtype(jn, first);
                else if (strncmp("Se", c, 2) == 0)
                    setmodtype(jn, second);
                else if (strncmp("Th", c, 2) == 0)
                    setmodtype(jn, third);
                else if (strncmp("Fo", c, 2) == 0)
                    setmodtype(jn, fourth);
                else
                    setmodtype(jn, last);
                
                entry = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo14"))->entry) );
                c = gtk_entry_get_text(entry);

                if (strncmp("Wee", c, 3) == 0)
                    setmodwhen(jn, weekday);
                else if (strncmp("Mon", c, 3) == 0)
                    setmodwhen(jn, monday);
                else if (strncmp("Tue", c, 3) == 0)
                    setmodwhen(jn, tuesday);
                else if (strncmp("Wed", c, 3) == 0)
                    setmodwhen(jn, wednesday);
                else if (strncmp("Thu", c, 3) == 0)
                    setmodwhen(jn, thursday);
                else if (strncmp("Fri", c, 3) == 0)
                    setmodwhen(jn, friday);
                else if (strncmp("Sat", c, 3) == 0)
                    setmodwhen(jn, saturday);
                else
                    setmodwhen(jn, sunday);

            }
        }


    }
    else
    {
        setrepeat(jn, off);
    }


    InsertNode(T, jn);
    SetAlarm();
    (void ) RebuildCList();

    if (button == GTK_BUTTON( lookup_widget(window1, "button20")) )
        on_button21_clicked((GtkButton *) NULL, (gpointer *) NULL);

    jn = (JobNode *) NULL;

}


void
on_button21_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{

    GtkWidget *but;

    but = GTK_WIDGET( lookup_widget( app1, "NewTask"));
    gtk_widget_set_sensitive(but, TRUE);
    but = GTK_WIDGET( lookup_widget( app1, "button25"));
    gtk_widget_set_sensitive(but, TRUE);

    if (window1 != (GtkWidget *) NULL)
        gtk_widget_hide(window1);

    gtk_widget_hide (window1);

}


void
on_window1_show                        (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_app1_show                           (GtkWidget       *widget,
                                        gpointer         user_data)
{
    FILE *fp;
    char *c;
    int i;
    GtkCList *cl;
    GtkStyle *s;
    GtkClock *clock1;



    if ( (c = getenv("HOME")) == (char *) NULL)
        c = (getpwuid(getuid()))->pw_dir;

    if (chdir(c) != 0)
    {
        fprintf(stderr, "Can't change directory to %s - aborting\n", c);
        app_destroy();
        gtk_exit(1);
    }

    if (chdir(".gnometd") != 0)
    {
        mkdir(".gnometd", 0755);
        if (chdir(".gnometd") != 0)
        {
            fprintf(stderr, "Can't change directory to ${HOME}/.gnometd - aborting\n");
            app_destroy();
            gtk_exit(1);
        }
    }

    clock1 = GTK_CLOCK(lookup_widget(app1, "clock1"));

    cl = GTK_CLIST ( lookup_widget(app1, "clist1")); 

    if ( (fp = fopen(CFGFILE, "r")) == (FILE *) NULL || fread(&Set, sizeof(settings), 1, fp) != 1)
    {
        strcpy(Set.dtfmt, "%d-%b-%y %H:%M");
        Set.fnt = gdk_font_load(DEFAULT_FONT);
        strncpy(Set.fntc, DEFAULT_FONT, 127);

        strcpy(Set.fname, "");

        Set.warned.red   = 0xffff;
        Set.warned.green = 0xffff;
        Set.warned.blue  = 0x0000;

        Set.expired.red   = 0xffff;
        Set.expired.green = 0x0000;
        Set.expired.blue  = 0x0000;

        Set.normal.red   = 0x0000;
        Set.normal.green = 0x0000;
        Set.normal.blue  = 0x0000;

        Set.background.red   = 0xbbbb;
        Set.background.green = 0xbbbb;
        Set.background.blue  = 0xbbbb;
    }
    else
    {
        Set.fnt = gdk_font_load(Set.fntc);
        if (Set.fnt == (GdkFont *) NULL)
            Set.fnt = gdk_font_load(DEFAULT_FONT);

        for (i = 0; i < 7; i++)
            gtk_clist_set_column_width(cl, i, Set.colwid[i]);
    }

    if ( fp != (FILE *) NULL )
        fclose(fp);

    fileselection1 = create_fileselection1 ();
    gtk_timeout_add(10000, (GtkFunction ) UpdateRemaining, (gpointer *) NULL);
    gtk_timeout_add(86400000, (GtkFunction ) UpdateDaily, (gpointer *) NULL);

    s = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(cl)));

    gdk_font_unref (s->font);
    s->font = Set.fnt;
    gdk_font_ref (Set.fnt); 

    gtk_widget_set_style(GTK_WIDGET(cl), s);

    gtk_clock_set_format (clock1, Set.dtfmt);
    gtk_clock_set_seconds(clock1, (time_t) 0);

    readtree(Set.fname);
    RebuildCList();
    SetAlarm();
    update_statusbar();

}

int readtree(char *openfile)
{
    JobNode *k;
    FILE *fp;
    int i,
        ct = 0;
    char *c;



    if (T == (JobTree *) NULL)
        T = TreeCreate();

    if (openfile[0] == '\0')
        return 0;

    if ( (fp = fopen(openfile, "r")) == (FILE *) NULL )
    {
        fprintf(stderr, "Can't open file %s\n", openfile);
        return -1;
    }


    while (!feof(fp))
    {
        k = NewNode(T);
        fread(&i, sizeof(int), 1, fp);

        if (feof(fp))
            break;

        c = malloc(i+1);
        fread(c, i, 1, fp);
        c[i] = '\0';

        if (fread((void *) k, sizeof(JobNode), 1, fp) != 1)
        {
            fprintf(stderr, "Possible corrupt input file %s\n", openfile);
            setjobdesc(k, c);
            jobfree(k);
            break;
        }
        else
        {
            setjobdesc(k, c);
            InsertNode(T, k);
        }

        ct += 1;
    }

    fclose(fp);

    return ct;


}

int writetree(char *savefile)
{
    FILE *fp;
    JobNode *j;
    int i,
        ct = 0;

    if ( (fp = fopen(savefile, "w")) == (FILE *) NULL )
    {
        fprintf(stderr, "Can't create/truncate '%s'\n", savefile);
        return -1;
    }

/*  fprintf(fp, "<?xml version=\"1.0\"?>\n");
    fprintf(fp, "<Gnome-ToDo>\n\n");
*/
    j = IntTreeWalk(T, (JobNode *) NULL);


    while (j != (JobNode *) NULL)
    {
        i = strlen(getjobdesc(j));
        if ( fwrite(&i, sizeof(int), 1, fp) != 1)
            fprintf(stderr, "Can't write output to output file correctly\n");

        if ( i > 0 && fwrite(getjobdesc(j), i, 1, fp) != 1)
            fprintf(stderr, "Can't write output to output file correctly\n");
        
        if ( fwrite(j, sizeof(JobNode), 1, fp) != 1)
            fprintf(stderr, "Can't write output to output file correctly\n");

        j = IntTreeWalk(T, j);

        ct += 1;
    }

/*  {   fprintf(fp, "<node>\n");
        fprintf(fp, "  <job_description>%s</job_description>\n", getjobdesc(j));
        c = xmljob(j);
        fwrite(c, sizeof(char), strlen(c), fp);
        fprintf(fp, "</node>\n\n");
        j = IntTreeWalk(T, j);

    fprintf(fp, "</Gnome-ToDo>\n"); */

    fclose(fp);

    return ct;
}


void
on_calendar2_day_selected              (GtkCalendar     *calendar,
                                        gpointer         user_data)
{
    DateChanged(calendar);
}

void
DateChanged     ( GtkCalendar *calendar )
{

    time_t t;
    struct tm st;

    gchar dstr[64];

    GtkEntry *dtentry,
             *hrent,
             *mient;

    dtentry  = GTK_ENTRY ( lookup_widget (window1, "entry3") );
    hrent    = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo1"))->entry) );
    mient    = GTK_ENTRY ( (GTK_COMBO (lookup_widget(window1, "combo3"))->entry) );

    gtk_calendar_get_date ( calendar,
                            (guint *) &st.tm_year, 
                            (guint *) &st.tm_mon, 
                            (guint *) &st.tm_mday);

    st.tm_hour = atoi((char *) gtk_entry_get_text(hrent));
    st.tm_min  = atoi((char *) gtk_entry_get_text(mient));
    st.tm_year  -= 1900;
    st.tm_isdst = -1;
    st.tm_sec  = 0;

    t =  mktime(&st);

    memcpy(&st, localtime(&t), sizeof(struct tm));

    strftime((char *) dstr, 64, Set.dtfmt, &st);
    gtk_entry_set_text( dtentry, dstr); 

    if (jn != (JobNode *) NULL)
        setjobtime(jn, t);

}


void
on_combo_entry1_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{

    DateChanged(GTK_CALENDAR (lookup_widget(window1, "calendar2")));

}


void
on_combo_entry3_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
    DateChanged(GTK_CALENDAR (lookup_widget(window1, "calendar2")));

}


void
on_button16_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
/*
    int i = 0;
    JobNode *j;
    GtkCList *cl;

    cl = GTK_CLIST(lookup_widget(app1, "clist1"));

    long j;

    srand48((long ) time((time_t) NULL));

    while ( i++ < 10000 )
    {
        jn = NewNode();
        j = lrand48();
        setjobtime(jn, (time_t ) i);
        setjobdesc(jn, "Chris Hall"); InsertNode(jn);
        printf("%10d\n", j);

    } 

    treewalk(T);

    while ( j = gtk_clist_get_row_data(cl, i++) )
        printf("%8d\n", (int ) j);
*/

    GtkWidget *but;

    static gboolean s = TRUE;

    but = lookup_widget(window1, "button22");

    gtk_widget_set_sensitive( but, s);
    s = !s;

    /* treewalk(T); */

}




void LoadMainWindow()
{
    GtkCList *cl;

    cl = GTK_CLIST ( lookup_widget (app1, "clist1"));

    (void ) RebuildCList();
}

void fill_date_example()
{
    gchar dstr[64];
    GtkEntry *dtfmtent,
             *dtexp;
    struct tm st;
    time_t t;

    dtfmtent = GTK_ENTRY ( (GTK_COMBO (lookup_widget(propertybox1, "combo2"))->entry) );
    dtexp    = GTK_ENTRY ( lookup_widget(propertybox1, "entry4"));

    t = time(&t);
    memcpy(&st, localtime(&t), sizeof(struct tm));
    strftime((char *) dstr, 64, gtk_entry_get_text(dtfmtent), &st);
    gtk_entry_set_text( dtexp, dstr);
}


void
on_combo_entry2_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
    fill_date_example();

    gnome_property_box_changed (GNOME_PROPERTY_BOX(propertybox1));

}


void
on_propertybox1_clicked                (GnomeDialog     *gnomedialog,
                                        gint             arg1,
                                        gpointer         user_data)
{

    printf("Property Box 1 Clicked\n");
}


gint UpdateRemaining(gpointer *p)
{
    GtkCList *cl;
    gint t, i = 0;
    time_t d, now;
    JobNode *j;
    char rstr[32];
    GdkColor col;
    gdouble pc;


    now = time((time_t *) NULL);

    cl = GTK_CLIST ( lookup_widget (app1, "clist1"));
    gtk_clist_freeze(cl);

    rstr[0] = '\0';

    while ( (j = (JobNode *) gtk_clist_get_row_data(cl, i)) != (JobNode *) NULL)
    {
        t = (gint ) getjobtime(j) - now;
        if ( t > 0 )
        {
            rstr[0] = '\0';
            d = t / 86400;
            if (d > 0)
            {
                sprintf(rstr, "%4d day%s ", (int ) d, d > 1 ? "s" : " ");
                t -= d * 86400;
            }
            else
                sprintf(rstr, "          ");

            d = t / 3600;
            if (d > 0 || rstr[5] == 'd')
            {
                sprintf(rstr, "%s%2d hour%s ", rstr, (int ) d, d == 1 ? " " : "s");
                t -= d * 3600;
            }
            else
                sprintf(rstr, "%s         ", rstr);

            d = t / 60;
            sprintf(rstr, "%s%2d minute%s", rstr, (int ) d, d == 1 ? " " : "s");

        }

        gtk_clist_set_text(cl, i, 3, rstr);

        switch (getjobaction(j))
        {
            case execute    : gtk_clist_set_text(cl, i, 4, "Execute");
                              break;

            case timer      : gtk_clist_set_text(cl, i, 4, "Timer");
                              break;

            case alarmclock : gtk_clist_set_text(cl, i, 4, "Alarm");
                              break;

            case timedout   : gtk_clist_set_text(cl, i, 4, "Expired");
                              break;

            case executed   : gtk_clist_set_text(cl, i, 4, "Executed");
                              break;
        }

        if ( (getjobtime(j) - now) <= 0)
            gtk_clist_set_foreground(cl, i, &Set.expired);
        else if ( (getwarntime(j) - now) <= 0 )
        {
            pc = ( (gdouble ) (now - getwarntime(j)) / (gdouble ) (getjobtime(j) - getwarntime(j)) ) * 100.0;

            col.red   = ( (Set.warned.red   - Set.normal.red)   * (gint ) pc) / 100 + Set.normal.red;
            col.green = ( (Set.warned.green - Set.normal.green) * (gint ) pc) / 100 + Set.normal.green;
            col.blue  = ( (Set.warned.blue  - Set.normal.blue)  * (gint ) pc) / 100 + Set.normal.blue;

            gtk_clist_set_foreground(cl, i, &col);
        }
        else
            gtk_clist_set_foreground(cl, i, &Set.normal);

        gtk_clist_set_background(cl, i, &Set.background);

        i += 1;
    }

    gtk_clist_thaw(cl);
    return 1;

}


gint RebuildCList()
{

    GtkCList *c;
    GtkScrolledWindow *sc;

    JobNode *j;

    char *line[7],
         wstr[64],
         dstr[64],
         rstr[64];

    time_t t;

    struct tm st;

    gint row;

    c = GTK_CLIST ( lookup_widget ( app1, "clist1"));
    sc = GTK_SCROLLED_WINDOW ( lookup_widget ( app1, "scrolledwindow2"));
    row = 0;


    gtk_clist_freeze(c);
    gtk_clist_clear(c);
    

    j = IntTreeWalk(T, (JobNode *) NULL);

    while (j != (JobNode *) NULL)
    {

        line[0] = getjobdesc(j);

        t = getjobtime(j);
        memcpy(&st, (void * ) localtime(&t), sizeof(struct tm));
        strftime((char *) dstr, 64, Set.dtfmt, &st);
        line[1] = dstr;
        
        if (getwarntime(j) != getjobtime(j))
        {
            t = getwarntime(j);
            memcpy(&st, localtime(&t), sizeof(struct tm));
            strftime((char *) wstr, 64, Set.dtfmt, &st);
            line[2] = wstr;
        }
        else
            line[2] = (char *) NULL;


        line[3] = (char *) NULL;
        line[4] = (char *) NULL;

        if ( getrepeat(j) == off )
            line[5] = (char *) NULL;
        else
        {
            t = getnextrun(j, no);
            memcpy(&st, (void * ) localtime(&t), sizeof(struct tm));
            strftime((char *) rstr, 64, Set.dtfmt, &st);
            line[5] = rstr; 
        }


        line[6] = (char *) NULL;

        gtk_clist_append(c, line);

        gtk_clist_set_row_data(c, row, (gpointer *) j);

        j = IntTreeWalk(T, j);
        row += 1;

    }


    UpdateRemaining((gpointer *) NULL);
    gtk_clist_thaw(c);

    return row;
}

void SetAlarm()
{
    time_t t;
    JobNode *j;

    if (alarmtag != 0)
    {
        gtk_timeout_remove(alarmtag);
        alarmtag = 0;
    }

    t = time((time_t *) NULL);

    j = IntTreeWalk(T, (JobNode *) NULL);

    while ( j != (JobNode *) NULL )
    {
        if ( (getjobtime(j) - t) <= 0 )
        {
            if ( getjobaction(j) != executed && getjobaction(j) != timedout)
                RunJob(j);
            j = IntTreeWalk(T, j);
        }
        else
        {
            if ( (getjobtime(j) - t) <= 90000 )
                alarmtag = gtk_timeout_add(((gint ) getjobtime(j) - (gint ) t) * 1000,
                                       (GtkFunction ) JobAction,
                                       (gpointer *) j);
            j = (JobNode *) NULL;
        }
    }

    RebuildCList();

}

gint JobAction(gpointer *j)
{
    alarmtag = 0;

    RunJob((JobNode *) j);
    SetAlarm();

    return 0;
}

void RunJob(JobNode *j)
{

    GtkWidget *w;
    GtkLabel *l;
    JobNode *k;
    time_t t;
    gchar lab[1024];
    struct tm *st;
    
    if ( ( t = getnextrun(j, yes)) != (time_t ) 0)
    {
        k = DupNode(T, j);
        setjobtime(k, t);
        setwarntime(k, t - (getjobtime(j) - getwarntime(j)) );
        InsertNode(T, k);
    }

    switch (getjobaction(j))
    {
        case timer      : setjobaction(j, timedout);
                          break;

        case alarmclock : setjobaction(j, timedout);
                          w = create_window2();
                          t = time((time_t *) NULL);
                          l = GTK_LABEL(lookup_widget(w, "label88"));
                          st = localtime(&t);
                          strftime((char *) lab, 64, Set.dtfmt, st);
                          strcat(lab, "\n\n");
                          strncat(lab, getjobdesc(j), 900);
                          gtk_label_set_text(l, lab);
                          gtk_widget_show(w);
                          break;
        
        case execute    : setjobaction(j, executed);
                          if ( fork() == (pid_t ) 0 )
                          {
                              fclose(stdin);
                              fclose(stdout);
                              fclose(stderr);
                              system(getjobdesc((JobNode *) j));
                              _exit(0);
                          }
                          break;

        default         : break;

    }


}
     

void
on_button23_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_button24_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkCList *cl;
    int i;

    cl = GTK_CLIST ( lookup_widget(app1, "clist1"));

    if (rowsel >= 0)
    {
        DeleteNode(T, (JobNode *) gtk_clist_get_row_data(cl, rowsel));

        if ( (i =  RebuildCList()) == 0 )
            rowsel = -1;
        else
        {
            if ( rowsel > i - 1)
                rowsel = i - 1;

            gtk_clist_select_row(cl, rowsel, 1);
        }
    }

}


void
on_clist1_select_row                   (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    rowsel = row;

}

gint UpdateDaily(gpointer *g)
{
    SetAlarm();
    return 1;
}


void
on_togglebutton1_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gtk_widget_set_sensitive(lookup_widget(window1, "frame5"), togglebutton->active);
}


void
on_repeatgroup_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    GtkWidget *w;
    GtkToggleButton *t;

    if (togglebutton->active == TRUE)
    {
        t = GTK_TOGGLE_BUTTON ( lookup_widget (window1, "radiobutton17"));
        if (t == togglebutton)
        {
            w = lookup_widget(window1, "frame3");
            gtk_widget_set_sensitive(w, TRUE);
            w = lookup_widget(window1, "frame4");
            gtk_widget_set_sensitive(w, FALSE);
        }
        else
        {
            w = lookup_widget(window1, "frame4");
            gtk_widget_set_sensitive(w, TRUE);
            w = lookup_widget(window1, "frame3");
            gtk_widget_set_sensitive(w, FALSE);
        }

    }

}


void
on_newfile_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkLabel *l;

    if ( dialog1 == NULL )
        dialog1 = create_dialog1();

    l = GTK_LABEL( lookup_widget(dialog1, "label87"));
    gtk_window_set_title(GTK_WINDOW(dialog1), user_data);
    gtk_object_set_user_data(GTK_OBJECT(dialog1), (gpointer ) user_data);
    gtk_label_set_text(l, "Save Current Tasks?");

    gtk_widget_show(dialog1);

}


void
on_file_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
    if ( fileselection1 == (GtkWidget *) NULL )
        fileselection1 = create_fileselection1();

    gtk_window_set_title(GTK_WINDOW(fileselection1), "Open");
    gtk_object_set_user_data(GTK_OBJECT(fileselection1), (gpointer ) "Open");

    gtk_widget_show(fileselection1);

}

void app_destroy()
{

}

void
on_save_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
    on_save1_activate ((GtkMenuItem *) NULL, (gpointer *) NULL);
}


void
on_propertybox1_show                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
    GtkWidget *w;
    GtkEntry *dtfmtent;



    dtfmtent = GTK_ENTRY ( (GTK_COMBO (lookup_widget(propertybox1, "combo2"))->entry) );

    w = lookup_widget(propertybox1, "fontpicker1");
    gnome_font_picker_set_font_name( GNOME_FONT_PICKER(w), Set.fntc);

    w = lookup_widget(propertybox1, "colorpicker5");
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(w), Set.warned.red, Set.warned.green, Set.warned.blue, 0);

    w = lookup_widget(propertybox1, "colorpicker6");
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(w), Set.expired.red, Set.expired.green, Set.expired.blue, 0);

    w = lookup_widget(propertybox1, "colorpicker7");
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(w), Set.normal.red, Set.normal.green, Set.normal.blue, 0);

    w = lookup_widget(propertybox1, "colorpicker8");
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(w), Set.background.red, Set.background.green, Set.background.blue, 0);

    gtk_signal_handler_block_by_func(GTK_OBJECT(dtfmtent), GTK_SIGNAL_FUNC(on_combo_entry2_changed), user_data);
    gtk_entry_set_text(dtfmtent, Set.dtfmt);
    gtk_signal_handler_unblock_by_func(GTK_OBJECT(dtfmtent), GTK_SIGNAL_FUNC(on_combo_entry2_changed), user_data);

    fill_date_example();

}


void
on_colorpicker5_color_set              (GnomeColorPicker *gnomecolorpicker,
                                        guint            arg1,
                                        guint            arg2,
                                        guint            arg3,
                                        guint            arg4,
                                        gpointer         user_data)
{
    gnome_property_box_changed (GNOME_PROPERTY_BOX(propertybox1));

}


void
on_fontpicker1_font_set                (GnomeFontPicker *gnomefontpicker,
                                        gchar            *arg1,
                                        gpointer         user_data)
{



    gnome_property_box_changed (GNOME_PROPERTY_BOX(propertybox1));

}


void
on_propertybox1_apply                  (GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data)
{
    GtkWidget *w;
    GtkEntry  *dtfmtent;
    GtkStyle  *s;
    GtkWidget *c;
    GtkClock  *clock1;

    clock1 = GTK_CLOCK(lookup_widget(app1, "clock1"));

    w = lookup_widget(propertybox1, "fontpicker1");
    strncpy(Set.fntc, gnome_font_picker_get_font_name(GNOME_FONT_PICKER(w)), 127);
    Set.fnt = gdk_font_load(Set.fntc);

    w = lookup_widget(propertybox1, "colorpicker5");
    gnome_color_picker_get_i16 ( GNOME_COLOR_PICKER(w), &Set.warned.red, &Set.warned.green, &Set.warned.blue,  (gushort *) NULL);

    w = lookup_widget(propertybox1, "colorpicker6");
    gnome_color_picker_get_i16 ( GNOME_COLOR_PICKER(w), &Set.expired.red, &Set.expired.green, &Set.expired.blue,  (gushort *) NULL);

    w = lookup_widget(propertybox1, "colorpicker7");
    gnome_color_picker_get_i16 ( GNOME_COLOR_PICKER(w), &Set.normal.red, &Set.normal.green, &Set.normal.blue,  (gushort *) NULL);

    w = lookup_widget(propertybox1, "colorpicker8");
    gnome_color_picker_get_i16 ( GNOME_COLOR_PICKER(w), &Set.background.red, &Set.background.green, &Set.background.blue,  (gushort *) NULL);

    dtfmtent = GTK_ENTRY ( (GTK_COMBO (lookup_widget(propertybox1, "combo2"))->entry) );

    strncpy(Set.dtfmt, gtk_entry_get_text(dtfmtent), 63);

    gtk_clock_set_format (GTK_CLOCK (clock1), Set.dtfmt);
    gtk_clock_set_seconds(clock1, (time_t) 0);

    c = lookup_widget(app1, "clist1");
    s = gtk_widget_get_style(c);
    
    gdk_font_unref (s->font);
    s->font = Set.fnt;
    gdk_font_ref (Set.fnt); 
    gtk_widget_set_style(c, s);

    RebuildCList();
}


void cfgfile_save()
{
    int i = 0;
    GtkCList *cl;
    FILE *fp;

    cl = GTK_CLIST(lookup_widget(app1, "clist1"));


    for (; i < 7; i++)
        Set.colwid[i] = cl->column[i].width;

    if ( (fp = fopen(CFGFILE, "w")) == (FILE *) NULL || fwrite(&Set, sizeof(settings), 1, fp) != 1)
        fprintf(stderr, "Failed to save configuration information correctly\n");
    else
        fclose(fp);



}


void
on_app1_destroy                        (GtkObject       *object,
                                        gpointer         user_data)
{
    if (Set.fname[0] == '\0')
        gtk_main_quit();
    else
        on_exit1_activate((GtkMenuItem *) NULL, (gpointer *) NULL);

}


void
on_button26_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkLabel *l;

    if ( dialog1 == NULL )
        dialog1 = create_dialog1();

    l = GTK_LABEL( lookup_widget(dialog1, "label87"));
    gtk_window_set_title(GTK_WINDOW(dialog1), "Quit?");
    gtk_object_set_user_data(GTK_OBJECT(dialog1), (gpointer ) "Quit?");
    gtk_label_set_text(l, "Exit GnomeTD");

    gtk_widget_show(dialog1);

    
}


void
on_button27_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    char *c;

    c = (char *) gtk_object_get_user_data( GTK_OBJECT (gtk_widget_get_toplevel(GTK_WIDGET(button)) ));

    gtk_widget_hide(dialog1);

    if (strcmp(c, "Quit?") == 0)
        on_exit1_activate((GtkMenuItem *) NULL, (gpointer *) NULL);
    else if (strcmp(c, "New") == 0)
    {
        if (Set.fname[0] != '\0')
        {
            writetree(Set.fname);
            DelTree(T, (JobNode *) NULL);
            Set.fname[0] = '\0';
            RebuildCList();
            SetAlarm();
            update_statusbar();
        }
        else
        {
            if ( fileselection1 == (GtkWidget *) NULL )
                fileselection1 = create_fileselection1();

            gtk_window_set_title(GTK_WINDOW(fileselection1), "New - Save As");
            gtk_object_set_user_data(GTK_OBJECT(fileselection1), (gpointer ) "New - Save As");

            gtk_widget_show(fileselection1);
        }
    }
    else if (strcmp(c, "Open") == 0)
    {
        if (Set.fname[0] != '\0')
        {
            writetree(Set.fname);
            DelTree(T, (JobNode *) NULL);
            if ( fileselection1 == (GtkWidget *) NULL )
                fileselection1 = create_fileselection1();

            gtk_window_set_title(GTK_WINDOW(fileselection1), "Open");
            gtk_object_set_user_data(GTK_OBJECT(fileselection1), (gpointer ) "Open");

            gtk_widget_show(fileselection1);
        }
        else
        {
            if ( fileselection1 == (GtkWidget *) NULL )
                fileselection1 = create_fileselection1();

            gtk_window_set_title(GTK_WINDOW(fileselection1), "Open - Save As");
            gtk_object_set_user_data(GTK_OBJECT(fileselection1), (gpointer ) "Open - Save As");

            gtk_widget_show(fileselection1);
        }
    }

            

}


void
on_button28_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    char *c;

    c = (char *) gtk_object_get_user_data( GTK_OBJECT( gtk_widget_get_toplevel(GTK_WIDGET(button)) ) );

    gtk_widget_hide(dialog1);

    if (strcmp(c, "New") == 0)
    {
        DelTree(T, (JobNode *) NULL);
        Set.fname[0] = '\0';
        RebuildCList();
        SetAlarm();
        update_statusbar();
    }
    else if (strcmp(c, "Open") == 0)
    {
        if ( fileselection1 == (GtkWidget *) NULL )
            fileselection1 = create_fileselection1();

        gtk_window_set_title(GTK_WINDOW(fileselection1), "Open");
        gtk_object_set_user_data(GTK_OBJECT(fileselection1), (gpointer ) "Open");

        gtk_widget_show(fileselection1);
    }


}


void
on_ok_button2_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    char *c;

    c = (char *) gtk_object_get_user_data( GTK_OBJECT( gtk_widget_get_toplevel(GTK_WIDGET(button)) ) );

    if (strcmp(c, "Save As") == 0)
    {
        strncpy(Set.fname, gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection1)), 255);
        gtk_widget_hide(fileselection1);
        writetree(Set.fname);
        cfgfile_save();
        update_statusbar();
    }
    else if (strcmp(c, "Quit - Save As") == 0)
    {
        strncpy(Set.fname, gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection1)), 255);
        gtk_widget_hide(fileselection1);
        writetree(Set.fname);
        cfgfile_save();
        gtk_main_quit();
    }
    else if (strcmp(c, "New - Save As") == 0)
    {
        strncpy(Set.fname, gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection1)), 255);
        gtk_widget_hide(fileselection1);
        writetree(Set.fname);
        cfgfile_save();
        DelTree(T, (JobNode *) NULL);
        Set.fname[0] = '\0';
        RebuildCList();
        SetAlarm();
        update_statusbar();
    }
    else if (strcmp(c, "Open") == 0)
    {
        strncpy(Set.fname, gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection1)), 255);
        gtk_widget_hide(fileselection1);
        DelTree(T, (JobNode *) NULL);
        readtree(Set.fname);
        cfgfile_save();
        RebuildCList();
        SetAlarm();
        update_statusbar();
    }
    else if (strcmp(c, "Open - Save As") == 0)
    {
        strncpy(Set.fname, gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection1)), 255);
        gtk_widget_hide(fileselection1);
        writetree(Set.fname);
        DelTree(T, (JobNode *) NULL);
        gtk_window_set_title(GTK_WINDOW(fileselection1), "Open");
        gtk_object_set_user_data( GTK_OBJECT(fileselection1), "Open");
        gtk_widget_show(fileselection1);
    }
}


void
on_cancel_button2_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    char *c;

    c = (char *) gtk_object_get_user_data( GTK_OBJECT( gtk_widget_get_toplevel(GTK_WIDGET(button)) ) );

    if (strcmp(c, "Quit - Save As") == 0)
    {
        Set.fname[0] = '\0';
        gtk_widget_hide(fileselection1);
        cfgfile_save();
        gtk_main_quit();
    }
    else
        gtk_widget_hide(fileselection1);

}


void
on_button29_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_hide(dialog1);

}


void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
    GtkWidget *but;


    window1 = (GtkWidget *) NULL;

    but = lookup_widget(app1, "button25");
    gtk_widget_set_sensitive(but, TRUE);
    but = lookup_widget(app1, "NewTask");
    gtk_widget_set_sensitive(but, TRUE);
}


void
on_fileselection1_destroy              (GtkObject       *object,
                                        gpointer         user_data)
{
    fileselection1 = (GtkWidget *) NULL;

}

void update_statusbar()
{
    GtkWidget *w;
    char s[1024];

    sprintf(s, "Current task list: ");

    w = lookup_widget(app1, "appbar1");

    if ( Set.fname[0] == '\0' )
        sprintf(s, "%sNew List", s);
    else
        sprintf(s, "%s%s", s, Set.fname);

    gnome_appbar_set_status(GNOME_APPBAR(w), s);
}

void
on__user_guide_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GnomeHelpMenuEntry help_ref = { "gnome-todo", "index.html" };

    gnome_help_display((void *) NULL, &help_ref);

}
