/*******************************************************************************
 * Module      : $RCSfile: jobnode.c,v $
 * Description : Module to keep track of the jobs and tasks, uses Red Black
 *             : Trees, which is probably a tad over engineered as I never
 *             : seem to have more than 20 odd taks in the list at any one time.
 *             :
 *             : The RBT code came from various places on the Internet, but most
 *             : sites seemed to take the code from Cormen, T.H.,
 *             : C.E. Leiserson, and R.L. Rivest. Introduction to Algorithms.
 *             :
 *             :
 * Date        : $Date: 2001/05/23 13:58:38 $
 * Version     : $Revision: 1.4 $
 * Author      : Chris Hall
 * Identifier  : $Id: jobnode.c,v 1.4 2001/05/23 13:58:38 chall Exp $
 ********************************************************************************
 * Change History
 *
 * $Log: jobnode.c,v $
 * Revision 1.4  2001/05/23 13:58:38  chall
 * Added GPL license text and installation instructions
 *
 * Revision 1.3  2001/05/23 12:30:00  chall
 * Release 0.1
 *
 ********************************************************************************
 *
 * Gnome ToDo - Task reminder program for Gnome
 * Copyright (C) 2001 Chris Hall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Chris Hall Chris@clapham.org
 */






#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "jobnode.h"
#include <time.h>

#define colour(y)      y->colour
#define left(y)        y->left
#define right(y)       y->right
#define parent(y)      y->parent

static void    RotateRight(JobTree *T, JobNode *x);
static void    RotateLeft(JobTree *T, JobNode *x);
static void    TreeInsert(JobTree *T, JobNode *node, JobNode *newnode);
static void    tw(JobTree *T, JobNode *j, int i, int k);
static JobNode *TreeMin(JobTree *T, JobNode *x);
static JobNode *Successor(JobTree *T, JobNode *x);
static void    fixtree(JobTree *T, JobNode *x);


extern GtkWidget *propertybox1;

void jobfree(JobNode *j)
{
    g_free(j->jobdesc);
    g_free(j);
}

JobTree *TreeCreate()
{
    JobTree *T;

    T       = malloc(sizeof(JobTree));
    T->nil  = malloc(sizeof(JobNode));
    T->root = malloc(sizeof(JobNode));

    left(T->nil)   = T->nil;
    right(T->nil)  = T->nil;
    parent(T->nil) = T->nil;
    colour(T->nil) = black;
    T->root = T->nil;

    return T;
    
}

void DelTree(JobTree *T, JobNode *j)
{
    if (j == (JobNode *) NULL)
    {
        DelTree(T, T->root);
        T->root = T->nil;
        return;
    }

    if (j == T->nil)
        return;

    DelTree(T, j->left);
    DelTree(T, j->right);
    jobfree(j);
}

JobNode *NewNode(JobTree *T)
{
    JobNode *j,
            *nil = T->nil;
    
    j = malloc(sizeof(JobNode));

    j->left      = nil;
    j->right     = nil;
    j->colour    = red;
    j->parent    = nil;
    j->jobtime   = (time_t ) 0;
    j->warntime  = (time_t ) 0;
    j->origday   = 0;
    j->action    = alarmclock;
    j->repeat    = off;

    return j;
}

JobNode *DupNode(JobTree *T, JobNode *j)
{
    JobNode *k;

    k = (JobNode *) malloc(sizeof(JobNode));
    memcpy(k, j, sizeof(JobNode));

    k->jobdesc = (char *) malloc((strlen(j->jobdesc) + 1) *sizeof(char));
    strcpy(k->jobdesc, j->jobdesc);

    k->left      = T->nil;
    k->right     = T->nil;
    k->colour    = red;
    k->parent    = T->nil;

    return k;
}


void setjobdesc(JobNode *j, char *desc)
{
    j->jobdesc = desc;
}

char *getjobdesc(JobNode *j)
{
    return j->jobdesc;
}

void setjobtime(JobNode *j, time_t start)
{
    j->jobtime = start;
}

void setwarntime(JobNode *j, time_t warn)
{
    j->warntime = warn;
}

void setorigdate(JobNode *j)
{
    struct tm *t;

    t = localtime(&j->jobtime);

    j->origday  = t->tm_mday;
    j->origmon  = t->tm_mon;
    j->origyear = t->tm_year;
}



void InsertNode(JobTree *T, JobNode *jn)
{
    JobNode *x = jn,
            *y;

    jn->left   = T->nil;
    jn->right  = T->nil;
    jn->parent = T->nil;
    jn->colour = red;

    if (T->root == T->nil)
        T->root = jn;
    else
    {
        TreeInsert(T, T->root, jn);

        while ( x != T->root && colour(parent(x)) == red )
        {
            if (parent(x) == left(parent(parent(x))) )
            {
                y = right(parent(parent(x)));
                if (colour(y) == red)
                {
                    colour(x->parent) = black;
                    colour(y) =  black;
                    colour(x->parent->parent) =  red;
                    x = parent(parent(x));
                }
                else
                {
                    if (x == right(parent(x)))
                    {
                        x = parent(x);
                        RotateLeft(T, x);
                    }
                    colour(x->parent) =  black;
                    colour(x->parent->parent) = red;
                    RotateRight(T, x->parent->parent);
                }
            }
            else
            {
                y = x->parent->parent->left;
                if (colour(y) == red)
                {
                    colour(x->parent) = black;
                    colour(y) = black;
                    colour(x->parent->parent) = red;
                    x = x->parent->parent;
                }
                else
                {
                    if (x == x->parent->left)
                    {
                        x = x->parent;
                        RotateRight(T, x);
                    }
                    colour(x->parent) = black;
                    colour(x->parent->parent) = red;
                    RotateLeft(T, x->parent->parent);
                }
            }

        }
    }

    colour(T->root) = black;
    
}

time_t getjobtime(JobNode *j)
{
    return j->jobtime;
}

time_t getwarntime(JobNode *j)
{
    return j->warntime;
}


static void RotateRight(JobTree *T, JobNode *x)
{
    JobNode *y;

    y = x->left;

    x->left = y->right;
    if ( y->right != T->nil)
        y->right->parent = x;

    y->parent = x->parent;

    if ( x->parent == T->nil )
        T->root = y;
    else
        if ( x == x->parent->right )
            x->parent->right = y;
        else
            x->parent->left = y;

    y->right = x;
    x->parent = y;
}

static void RotateLeft(JobTree *T, JobNode *x)
{
    JobNode *y;

    y = x->right;

    x->right = y->left;
    if ( y->left != T->nil )
        y->left->parent = x;

    y->parent = x->parent;

    if ( x->parent == T->nil )
        T->root = y;
    else
        if ( x == x->parent->left )
            x->parent->left = y;
        else
            x->parent->right = y;

    y->left = x;
    x->parent = y;
}

static void TreeInsert(JobTree *T, JobNode *node, JobNode *newnode)
{

    if ( newnode->jobtime <= node->jobtime )
        if (node->left == T->nil)
        {
            node->left      = newnode;
            newnode->parent = node;
        }
        else
        {
            TreeInsert(T, node->left, newnode);
        }
    else
        if (node->right == T->nil)
        {
            node->right = newnode;
            newnode->parent = node;
        }
        else
        {
            TreeInsert(T, node->right, newnode);
        }
}


void treewalk(JobTree *T)
{

    printf("\n\n\n\n");
    tw(T, T->root, 0, 0);
}

static void tw(JobTree *T, JobNode *j, int i, int k)
{
    time_t p, o;
    char c[30], d[30];

    if (j != T->nil)
    {
        p = getjobtime(j);
        o = getwarntime(j);

        strcpy(c, ctime(&p));
        strcpy(d, ctime(&o));

        c[strlen(c) - 1] = '\0';
        d[strlen(d) - 1] = '\0';

        tw(T, j->left, i + 1, 0);
        printf("%2d %s %s: %s %s %s - %s\n", i,
                                       k == 1 ? "right" : "left ",
                                       j->colour == red ? "red  " : "black",
                                       c,
                                       left(j) == T->nil ? "nil" : "---",
                                       right(j) == T->nil ? "nil" : "---",
                                       j->jobdesc);
        tw(T, j->right, i + 1, 1);
    }
}


void setjobaction(JobNode *j, jobact a)
{
    j->action = a;
}


jobact getjobaction(JobNode *j)
{
    return j->action;
}


static JobNode *TreeMin(JobTree *T, JobNode *x)
{
     while (x->left != T->nil)
         x = x->left;
     return x;
}

static JobNode *Successor(JobTree *T, JobNode *x)
{
    JobNode *y;

    if (x->right != T->nil)
        return TreeMin(T, x->right);

    y = x->parent;

    while (y != T->nil && x == y->right)
    {
        x = y;
        y = x->parent;
    }

    return y;
}

    
void DeleteNode(JobTree *T, JobNode *z)
{
    JobNode *x, *y, *nil = T->nil;


    if (left(z) == nil || right(z) == nil)
        y = z;
    else
        y = Successor(T, z);


    if (left(y) != nil)
        x = left(y);
    else
        x = right(y);

    parent(x) = parent(y);


    if (parent(y) == nil)
        T->root = x;
    else
        if ( y == left(parent(y)) )
            left(parent(y)) = x;
        else
            right(parent(y)) = x;

    if ( y != z )
    {
        g_free((gpointer *) getjobdesc(z));
        setjobdesc(z, getjobdesc(y));
        setjobtime(z, getjobtime(y));
        setwarntime(z, getwarntime(y));
        setjobaction(z, getjobaction(y));
        if ( colour(y) == black )
            fixtree(T, x);
        g_free(y);
    }
    else
    {
        if ( colour(y) == black )
            fixtree(T, x);
        jobfree(y);
    }

    parent(T->nil) = T->nil;


}

static void fixtree(JobTree *T, JobNode *x)
{
    JobNode *w;

    while (x != T->root && colour(x) == black)
    {
        if ( x == left(parent(x)) )
        {
            w = right(parent(x));
            if (colour(w) == red)
            {
                colour(w) = black;
                colour(parent(x)) = red;
                RotateLeft(T, parent(x));
                w = right(parent(x));
            }

            if ( colour(left(w)) == black && colour(right(w)) == black )
            {
                colour(w) = red;
                x = parent(x);
            }
            else
            {
                if ( colour(right(w)) == black )
                {
                    colour(left(w)) = black;
                    colour(w) = red;
                    RotateRight(T, w);
                    w = right(parent(x));
                }

                colour(w) = colour(parent(x));
                colour(parent(x)) = black;
                colour(right(w)) = black;
                RotateLeft(T, parent(x));

                x = T->root;
            }
        }
        else
        {
            w = left(parent(x));
            if (colour(w) == red)
            {
                colour(w) = black;
                colour(parent(x)) = red;
                RotateRight(T, parent(x));
                w = left(parent(x));
            }

            if ( colour(right(w)) == black && colour(left(w)) == black )
            {
                colour(w) = red;
                x = parent(x);
            }
            else
            {
                if ( colour(left(w)) == black )
                {
                    colour(right(w)) = black;
                    colour(w) = red;
                    RotateLeft(T, w);
                    w = left(parent(x));
                }

                colour(w) = colour(parent(x));
                colour(parent(x)) = black;
                colour(left(w)) = black;
                RotateRight(T, parent(x));

                x = T->root;
            }
        }
    }

    colour(x) = black;
}

JobNode *IntTreeWalk(JobTree *T, JobNode *x)
{
    if (x == (JobNode *) NULL)
    {
        x = T->root;

        if ( x == T->nil )
            return (JobNode *) NULL;

        while ( left(x) != T->nil)
            x = left(x);

        return x;
    }

    if ( (x = Successor(T, x)) == T->nil)
        return (JobNode *) NULL;
    else
        return x;
}

void setrepeat(JobNode *j, rep y)
{
    j->repeat = y;
}

rep getrepeat(JobNode *j)
{
    return j->repeat;
}

void setrepeaton(JobNode *j, repwhen r, yesno n)
{
    switch (r)
    {
        case sunday:      j->repeatinfo.daily.days[0] = n;
                          break;

        case monday:      j->repeatinfo.daily.days[1] = n;
                          break;

        case tuesday:     j->repeatinfo.daily.days[2] = n;
                          break;

        case wednesday:   j->repeatinfo.daily.days[3] = n;
                          break;

        case thursday:    j->repeatinfo.daily.days[4] = n;
                          break;

        case friday:      j->repeatinfo.daily.days[5] = n;
                          break;

        case saturday:    j->repeatinfo.daily.days[6] = n;
                          break;

        case hour:        j->repeatinfo.daily.hour = n;
                          break;

        default:          break;
    }

}

yesno   getrepeaton(JobNode *j, repwhen r)
{
    switch (r)
    {
        case sunday:      return j->repeatinfo.daily.days[0];

        case monday:      return j->repeatinfo.daily.days[1];

        case tuesday:     return j->repeatinfo.daily.days[2];

        case wednesday:   return j->repeatinfo.daily.days[3];

        case thursday:    return j->repeatinfo.daily.days[4];

        case friday:      return j->repeatinfo.daily.days[5];

        case saturday:    return j->repeatinfo.daily.days[6];

        case hour:        return j->repeatinfo.daily.hour;

        default:          return no;
    }
}



void setstarton(JobNode *j, gint start)
{
    j->repeatinfo.daily.start = start;
}

gint getstarton(JobNode *j)
{
    return j->repeatinfo.daily.start;
}

void setendon(JobNode *j, gint end)
{
    j->repeatinfo.daily.end = end;
}

gint getendon(JobNode *j)
{
    return j->repeatinfo.daily.end;
}

void setmodtype(JobNode *j, rel r)
{
    j->repeatinfo.mthyr.relative = r;
}

rel getmodtype(JobNode *j)
{
    return j->repeatinfo.mthyr.relative;
}

void setmodwhen(JobNode *j, repwhen r)
{
    j->repeatinfo.mthyr.day = r;
}

repwhen getmodwhen(JobNode *j)
{
    return j->repeatinfo.mthyr.day;
}

void setsameday(JobNode *j, yesno y)
{
    j->repeatinfo.mthyr.sameday = y;
}

yesno getsameday(JobNode *j)
{
    return j->repeatinfo.mthyr.sameday;
}

time_t getnextrun(JobNode *j, yesno update)
{
    struct tm or;
    time_t t, n;
    int i,
        d,
        lmth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
    yesno dayset = no;

    if (j->repeat == off)
        return (time_t ) 0;

    t = time((time_t *) NULL);


    if (j->repeat == daily)
    {
        n = getjobtime(j);
        for (i = 0; i <= 6; i++)
            if (j->repeatinfo.daily.days[i] == yes)
                dayset = yes;

        if ( dayset == no )
            return (time_t ) 0;

        if (j->repeatinfo.daily.hour == yes)
        {
            n += 3600;
            memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
            if ( or.tm_hour > j->repeatinfo.daily.end)
            {
                do
                {
                    n += 3600;
                    memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
                } while ( or.tm_hour < j->repeatinfo.daily.start || or.tm_hour > j->repeatinfo.daily.end);

            }

            while (j->repeatinfo.daily.days[or.tm_wday] == no)
            {
                n += 86400;
                memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
            }
        }
        else
            do
            {
                n += 86400;
                memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
            } while (j->repeatinfo.daily.days[or.tm_wday] == no);
        
    }
    else
    {
        memcpy((void *) &or, (void *) localtime(&j->jobtime), sizeof(or));

        or.tm_year = j->origyear;
        or.tm_mon  = j->origmon;

        if (j->repeat == monthly)
        {

            or.tm_mon = or.tm_mon + 1;
            if (or.tm_mon > 12)
            {
                or.tm_mon = 0;
                or.tm_year += 1;
            }

        }
        else
            or.tm_year += 1;

        if ( or.tm_year % 4 == 0 )
        {
            lmth[1] = 29;
            if ( or.tm_year % 100 == 0 )
                lmth[1] = 28;
            if ( or.tm_year % 400 == 0 )
                lmth[1] = 29;
        }

        if (j->origday > lmth[or.tm_mon])
            or.tm_mday = lmth[or.tm_mon];
        else
            or.tm_mday = j->origday;
        
        or.tm_isdst = -1;

        n = mktime(&or);
        memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
        if (update == yes)
        {
            j->origmon  = or.tm_mon;
            j->origyear = or.tm_year;
        }

        i = 0;

        if ( j->repeatinfo.mthyr.sameday == no )
            switch (j->repeatinfo.mthyr.relative)
            {
                case previous :    if ( j->repeatinfo.mthyr.day == weekday)
                                       while (or.tm_wday == 6 || or.tm_wday == 0)
                                       {
                                           n -= 86400;
                                           memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
                                       }
                                   else
                                       while (or.tm_wday != j->repeatinfo.mthyr.day)
                                       {
                                           n -= 86400;
                                           memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
                                       }

                                   break;

                case next     :    if ( j->repeatinfo.mthyr.day == weekday)
                                       while (or.tm_wday == 6 || or.tm_wday == 0)
                                       {
                                           n += 86400;
                                           memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
                                       }
                                   else
                                       while (or.tm_wday != j->repeatinfo.mthyr.day)
                                       {
                                           n += 86400;
                                           memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
                                       }

                                   break;

                case fourth   :    i += 1;
                case third    :    i += 1;
                case second   :    i += 1;
                                   if (j->repeatinfo.mthyr.day == weekday)
                                       break;
                                   
                case first    :    or.tm_mday = 1;
                                   n = mktime(&or);
                                   memcpy((void *) &or, (void *) localtime(&n), sizeof(or));

                                   if (j->repeatinfo.mthyr.day == weekday)
                                   {
                                       if (or.tm_wday == 0)
                                           n += 86400;
                                       else if (or.tm_wday == 6)
                                           n += 172800;
                                   }
                                   else
                                   {
                                       d = j->repeatinfo.mthyr.day - or.tm_wday;
                                       if (d < 0)
                                           d += 7;
                                       n += d * 86400 +  604800 * i;
                                   }

                                   memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
                                   break;

                case last     :    or.tm_mday = lmth[or.tm_mon];
                                   n = mktime(&or);
                                   memcpy((void *) &or, (void *) localtime(&n), sizeof(or));

                                   if (j->repeatinfo.mthyr.day == weekday)
                                   {
                                       if (or.tm_wday == 6)
                                           n -= 86400;
                                       else if (or.tm_wday == 0)
                                           n -= 172800;
                                   }
                                   else
                                   {
                                       d = or.tm_wday - j->repeatinfo.mthyr.day;
                                       if (d < 0)
                                           d += 7;
                                       n -= d * 86400;
                                   }

                                   memcpy((void *) &or, (void *) localtime(&n), sizeof(or));
                                   break;


            }

        }

                                       
    n = mktime(&or);


    return n;


}

char *xmljob(JobNode *j)
{
    static char c[1024];
    static char *d[] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

    int i;


    sprintf(c, "  <start_time>%ld</starttime>\n", (long ) getjobtime(j));
    sprintf(c, "%s  <warn_time>%ld</warntime>\n", c, (long ) getwarntime(j));
    sprintf(c, "%s  <action>", c);
    
    switch (getjobaction(j))
    {
        case alarmclock :    sprintf(c, "%salarm", c); break;
        case timer      :    sprintf(c, "%stimer", c); break;
        case execute    :    sprintf(c, "%sexecute", c); break;
        case timedout   :    sprintf(c, "%stimedout", c); break;
        case executed   :    sprintf(c, "%sexecuted", c); break;
    }

    sprintf(c, "%s</action>\n", c);
    
    sprintf(c, "%s  <day_of_month>%d</day_of_month>\n", c, j->origday);
    sprintf(c, "%s  <month_run>%d</month_run>\n", c, j->origmon);
    sprintf(c, "%s  <year_run>%d</year_run>\n", c, j->origyear);
            
    switch (j->repeat)
    {
        case daily   :   sprintf(c, "%s  <repeat>\n", c);
                         sprintf(c, "%s    <type>daily</type>\n", c);
                         sprintf(c, "%s    <repeat_days>\n", c);
                         for (i = 0; i < 7; i++)
                             if ( j->repeatinfo.daily.days[i] == yes )
                                 sprintf(c, "%s      <day>%s</day>\n", c, d[i]);
                         sprintf(c, "%s    </repeat_days>\n", c);
                         sprintf(c, "%s    <repeat_hour on=\"%s\">\n", c,
                                     j->repeatinfo.daily.hour == yes ? "yes" : "no");
    
                         if ( j->repeatinfo.daily.hour == yes )
                         {
                             sprintf(c, "%s      <hourly_start>%d</hourly_start>\n", c, j->repeatinfo.daily.start);
                             sprintf(c, "%s      <hourly_end>%d</hourly_end>\n", c, j->repeatinfo.daily.end);
                         }

                         sprintf(c, "%s    </repeat_hour>\n", c);
                         break;

        case monthly  :
        case yearly   : sprintf(c, "%s  <repeat>\n", c);
                        sprintf(c, "%s    <type>%s</type>\n", c, j->repeat == monthly ? "monthly" : "yearly");
                        sprintf(c, "%s    <repeat_modifier>\n", c);
                        sprintf(c, "%s      <sameday>%s</sameday>\n", c,
                                    j->repeatinfo.mthyr.sameday == yes ? "yes" : "no");

                        if ( j->repeatinfo.mthyr.sameday == no );
                        {
                            sprintf(c, "%s      <relative>", c);

                            switch(j->repeatinfo.mthyr.relative)
                            {
                                case previous : sprintf(c, "%sPrevious", c); break;
                                case next     : sprintf(c, "%sNext", c); break;
                                case first    : sprintf(c, "%sFirst", c); break;
                                case second   : sprintf(c, "%sSecond", c); break;
                                case third    : sprintf(c, "%sThird", c); break;
                                case fourth   : sprintf(c, "%sFourth", c); break;
                                case last     : sprintf(c, "%sLast", c); break;
                            }

                            sprintf(c, "%s</relative>\n", c);

                            sprintf(c, "%s      <when>", c);

                            switch(j->repeatinfo.mthyr.day)
                            {
                                case sunday    : sprintf(c, "%sSunday", c); break;
                                case monday    : sprintf(c, "%sMonday", c); break;
                                case tuesday   : sprintf(c, "%sTuesday", c); break;
                                case wednesday : sprintf(c, "%sWednesday", c); break;
                                case thursday  : sprintf(c, "%sThursday", c); break;
                                case friday    : sprintf(c, "%sFriday", c); break;
                                case saturday  : sprintf(c, "%sSaturday", c); break;
                                case weekday   : sprintf(c, "%sWeekday", c); break;
                                default        : break;
                            }
                            sprintf(c, "%s</when>\n", c);

                        }

                        sprintf(c, "%s    </repeat_modifier>", c);

                        break;

        case off      : sprintf(c, "%s  <repeat>\n", c);
                        sprintf(c, "%s    <type>off</type>\n", c);    
                        break;
    }
                                 
    sprintf(c, "%s  </repeat>\n", c);

    return c;
}

