/*******************************************************************************
 * Module      : $RCSfile: jobnode.h,v $
 * Description : Header file for jobnode.c, a Reb/Black Tree binary tree
 *             : implementation to keep the list of tasks
 *             :
 * Date        : $Date: 2001/05/23 12:30:00 $
 * Version     : $Revision: 1.2 $
 * Author      : Chris Hall
 * Identifier  : $Id: jobnode.h,v 1.2 2001/05/23 12:30:00 chall Exp $
 ********************************************************************************
 * Change History
 *
 * $Log: jobnode.h,v $
 * Revision 1.2  2001/05/23 12:30:00  chall
 * Release 0.1
 *
 ********************************************************************************/



#include <gnome.h>

typedef enum {red, black} col;
typedef enum {timer, alarmclock, execute, executed, timedout} jobact;
typedef enum {off, daily, monthly, yearly} rep;
typedef enum {sunday, monday, tuesday, wednesday, thursday, friday, saturday, hour, weekday} repwhen;
typedef enum {previous, next, first, second, third, fourth, last } rel;
typedef enum {yes, no} yesno;

typedef struct __JobNode {
    time_t  jobtime,
            warntime;

    char    *jobdesc;

    struct __JobNode *left,
                     *right,
                     *parent;

    col    colour;
    jobact action;

    int origday,
        origmon,
        origyear;

    rep repeat;

    union {
        struct _daily
        {
            yesno days[7];
            yesno hour;
            int   start,
                  end;
        } daily;
        struct _mthyr
        {
            yesno    monthyear;
            yesno   sameday;
            rel     relative;
            repwhen day;
        } mthyr;
    } repeatinfo;

} JobNode;


typedef struct __JobTree {
    JobNode *nil,
            *root;
} JobTree;


void       jobfree       (JobNode *);
JobTree    *TreeCreate   (void);
void       DelTree       (JobTree *, JobNode *);
JobNode    *NewNode      (JobTree *);
JobNode    *DupNode      (JobTree *, JobNode *);
void       setjobdesc    (JobNode *, char *);
char       *getjobdesc   (JobNode *);
void       setjobtime    (JobNode *, time_t );
void       setwarntime   (JobNode *, time_t );
void       setrepeat     (JobNode *, rep );
void       setrepeaton   (JobNode *, repwhen, yesno );
rep        getrepeat     (JobNode *);
yesno      getrepeaton   (JobNode *, repwhen);
void       InsertNode    (JobTree *, JobNode *);
time_t     getjobtime    (JobNode *);
time_t     getwarntime   (JobNode *);
void       treewalk      (JobTree *);
void       setjobaction  (JobNode *, jobact );
jobact     getjobaction  (JobNode *);
void       DeleteNode    (JobTree *, JobNode *);
JobNode    *IntTreeWalk  (JobTree *, JobNode *);
void       setstarton    (JobNode *, gint);
void       setendon      (JobNode *, gint);
gint       getstarton    (JobNode *);
gint       getendon      (JobNode *);
void       setmodtype    (JobNode *, rel);
void       setmodwhen    (JobNode *, repwhen);
rel        getmodtype    (JobNode *);
repwhen    getmodwhen    (JobNode *);
void       setsameday    (JobNode *, yesno);
yesno      getsameday    (JobNode *);
time_t     getnextrun    (JobNode *, yesno);
void       setorigdate   (JobNode *);
char       *xmljob       (JobNode *);
